/*******************************************************************@CM2CPYR*/
/*                                                                  @CM2CPYR*/
/*     Licensed Materials - Property of IBM                         @CM2CPYR*/
/*                                                                  @CM2CPYR*/
/*     5622-078                                                     @CM2CPYR*/
/*     (C) Copyright IBM Corp. 1988, 1995 All Rights Reserved.      @CM2CPYR*/
/*                                                                  @CM2CPYR*/
/*     US Government Users Restricted Rights - Use, duplication or  @CM2CPYR*/
/*     disclosure restricted by GSA ADP Schedule Contract with      @CM2CPYR*/
/*     IBM Corp.                                                    @CM2CPYR*/
/*                                                                  @CM2CPYR*/
/*******************************************************************@CM2CPYR*/
#ifdef __cplusplus
  extern "C" {
#endif

#ifndef  APPC_C_INCLUDED
#define  APPC_C_INCLUDED

#ifndef  APPCDEF_INCLUDED
#include <appcdef.h>
#endif

/****************************************************************************/
/* Common Verb Header                                                       */
/****************************************************************************/


typedef struct appc32_header {
    USHORT opcode;                     /* Verb operation code               */
    UCHAR  opext;                      /* Verb extension code               */
    UCHAR  reserv1[5];                 /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR  reserv2[2];                 /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
} APPC32_HEADER;

#define  AP_OPEXT_MAPPED_CONVERSATION 0x0001 /* Mapped Conversation         */
#define  AP_OPEXT_NON_BLOCKING        0x0002 /* Non Blocking                */
#define  AP_OPEXT_FULL_DUPLEX         0x0004 /* Full Duplex                 */
#define  AP_OPEXT_INCOMPLETE          0x0040 /* Incomplete                  */
#define  AP_OPEXT_MAPPED_AND_NON_BLOCKING      AP_OPEXT_MAPPED_CONVERSATION | \
                                               AP_OPEXT_NON_BLOCKING
#define  AP_OPEXT_MAPPED_AND_FULL_DUPLEX       AP_OPEXT_MAPPED_CONVERSATION | \
                                               AP_OPEXT_FULL_DUPLEX
#define  AP_OPEXT_NON_BLOCKING_AND_FULL_DUPLEX AP_OPEXT_NON_BLOCKING | \
                                               AP_OPEXT_FULL_DUPLEX
#define  AP_OPEXT_MAPPED_NON_BLOCKING_AND_FULL_DUPLEX  \
                                               AP_OPEXT_MAPPED_CONVERSATION | \
                                               AP_OPEXT_NON_BLOCKING |  \
                                               AP_OPEXT_FULL_DUPLEX

/****************************************************************************/
/* Application Control Interface                                            */
/****************************************************************************/

/*--------------------------------------------------------------------------*/
/* TP_NAME_ENTRY is used to describe one name within the TP_NAME_LIST       */
/* pointed to by a RECEIVE_ALLOCATE VCB with the use_name_list flag set     */
/* to AP_YES.                                                               */
/*--------------------------------------------------------------------------*/
typedef struct tp_name_entry {         /* Receive_Allocate(mult) name entry */
    ULONG name_length;                 /* Name length                       */
    ULONG name_wildcard;               /* Wildcard flag                     */
    UCHAR name;                        /* First character of name           */
} TP_NAME_ENTRY;

/*--------------------------------------------------------------------------*/
/* TP_NAME_LIST is the header of the name list pointed to by a              */
/* RECEIVE_ALLOCATE with the use_name_list flag set to AP_YES. It is        */
/* is followed by one or more TP_NAME_ENTRIES.                              */
/*--------------------------------------------------------------------------*/
typedef struct tp_name_list {          /* Receive_Allocate(mult) name list  */
    ULONG name_count;                  /* Number of names in the list       */
    TP_NAME_ENTRY name_entry;          /* The first entry in the list       */
} TP_NAME_LIST;


typedef struct receive_allocate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_RECEIVE_ALLOCATE            */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR sync_level;                  /* Sync Level                        */
                                       /*    AP_NONE                        */
                                       /*    AP_CONFIRM                     */
                                       /*    AP_SYNCPT                      */
    UCHAR conv_type;                   /* Conversation type                 */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_MAPPED_CONVERSATION         */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII)                  */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR reserv3[4];                  /* Reserved                          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR pip_incoming;                /* PIP data coming on next RCV&WAIT  */
    UCHAR conversation_style;          /* Conversation Style                */
                                       /*    AP_TWO_WAY_ALTERNATE           */
                                       /*    AP_TWO_WAY_SIMULTANEOUS        */
    UCHAR syncpt_enabled;              /* Whether TP is enabled for         */
                                       /* syncpt via linking to a           */
                                       /* syncpt provider:                  */
                                       /*    AP_YES, AP_NO                  */
    UCHAR use_name_list;               /* Receive_Allocate(multiple)        */
                                       /*    AP_YES, AP_NO                  */
                                       /*    Indicates whether name_list_ptr*/
                                       /*    pointer to list of names.      */
    UCHAR reserv4[3];                  /* Reserved                          */
    TP_NAME_LIST *name_list_ptr;       /* Name list pointer if              */
                                       /* Receive_Allocate(multiple)        */
    ULONG name_list_length;            /* Length of name list               */
    UCHAR reserv5[32];                 /* Reserved                          */
} RECEIVE_ALLOCATE;


typedef struct tp_ended {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_TP_ENDED                    */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR type;                        /* Type of TP Ended                  */
                                       /*    AP_SOFT                        */
                                       /*    AP_HARD                        */
    UCHAR reserv4[35];                 /* Reserved                          */
} TP_ENDED;


typedef struct tp_started {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_TP_STARTED                  */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_EXTENDED_VERB               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII) or               */
                                       /* default LU (all binary zeros)     */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR syncpt_enabled;              /* Whether TP is enabled for         */
                                       /* syncpt via linking to a           */
                                       /* syncpt provider:                  */
                                       /*    AP_YES, AP_NO                  */
    UCHAR reserv4[35];                 /* Reserved                          */
} TP_STARTED;

/****************************************************************************/
/* Transaction Programming Interface - Basic Conversation Verbs             */
/****************************************************************************/

typedef struct allocate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_ALLOCATE                  */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR conv_type;                   /* Conversation type                 */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_MAPPED_CONVERSATION         */
    UCHAR sync_level;                  /* Sync Level                        */
                                       /*    AP_NONE                        */
                                       /*    AP_CONFIRM                     */
                                       /*    AP_SYNCPT                      */
    UCHAR reserv3[2];                  /* Reserved                          */
    UCHAR rtn_ctl;                     /* Return Control                    */
                                       /*    AP_WHEN_SESSION_ALLOCATED      */
                                       /*    AP_IMMEDIATE                   */
                                       /*    AP_WHEN_SESSION_FREE           */
                                       /*    AP_WHEN_CONV_GROUP_ALLOC       */
                                       /*    AP_WHEN_CONWINNER_ALLOC        */
    UCHAR conversation_style;          /* Conversation Style                */
                                       /*    AP_TWO_WAY_ALTERNATE           */
                                       /*    AP_TWO_WAY_SIMULTANEOUS        */
    UCHAR reserv4[2];
    ULONG conv_group_id;               /* Conversation Group ID             */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR security;                    /* Security                          */
                                       /*    AP_NONE                        */
                                       /*    AP_SAME                        */
                                       /*    AP_PGM                         */
    UCHAR reserv5[11];                 /* Reserved                          */
    UCHAR pwd[10];                     /* Password (EBCDIC type AE)         */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG pip_dlen;                    /* PIP Data Length (0-32767)         */
                                       /*    (0 if no pip data)             */
    UCHAR * pip_dptr;                  /* PIP Data address                  */
    UCHAR reserv6;                     /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv7[34];                 /* Reserved                          */
} ALLOCATE;


typedef struct confirm {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_CONFIRM                   */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[18];                 /* Reserved                          */
} CONFIRM;


typedef struct confirmed {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_CONFIRMED                 */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} CONFIRMED;


typedef struct deallocate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_DEALLOCATE                */
    UCHAR opext;                       /* Verb extension code               */
                                       /* Calculate value using             */
                                       /*  instructions in APPCDEF.H -      */
                                       /*   Include AP_EXTENDED_VERB        */
                                       /*   in the calculated value.        */
                                       /*   May or may not include          */
                                       /*    AP_FDX_VERB and/or             */
                                       /*    AP_NONBLOCKING_VERB            */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR dealloc_type;                /* Type of Deallocate                */
                                       /*    AP_SYNC_LEVEL                  */
                                       /*    AP_FLUSH                       */
                                       /*    AP_ABEND_PROG                  */
                                       /*    AP_ABEND_SVC                   */
                                       /*    AP_ABEND_TIMER                 */
                                       /*    AP_CANCEL                      */
    UCHAR reserv3[2];                  /* Reserved                          */
    ULONG log_dlen;                    /* Log Data Length (0 to 32767)      */
                                       /*    (0 if no log data)             */
    UCHAR * log_dptr;                  /* Log Data address                  */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR reserv4[16];                 /* Reserved                          */
} DEALLOCATE;


typedef struct flush {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_FLUSH                     */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} FLUSH;


typedef struct get_attributes {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_GET_ATTRIBUTES            */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv4;                     /* Reserved                          */
    UCHAR sync_level;                  /* Sync Level                        */
                                       /*    AP_NONE                        */
                                       /*    AP_CONFIRM                     */
                                       /*    AP_SYNCPT                      */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR net_name[8];                 /* NETID (EBCDIC type A)             */
    UCHAR lu_name[8];                  /* LU Name (EBCDIC type A)           */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII)                  */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR plu_un_name[8];              /* Partner LU Uninterpreted Name     */
                                       /*    (EBCDIC type AE)               */
    UCHAR reserv5[2];                  /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv6;                     /* Reserved                          */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    UCHAR  conv_corr_len;              /* Coversation correlator length     */
    UCHAR  conv_corr[8];               /* Conversation correlator           */
    UCHAR conversation_state;          /* Conversation State                */
                                       /*    AP_RESET_STATE                 */
                                       /*    AP_SEND_STATE                  */
                                       /*    AP_RECEIVE_STATE               */
                                       /*    AP_CONFIRM_STATE               */
                                       /*    AP_CONFIRM_SEND_STATE          */
                                       /*    AP_CONFIRM_DEALL_STATE         */
                                       /*    AP_PEND_DEALL_STATE            */
                                       /*    AP_END_CONV_STATE              */
                                       /*    AP_TP_VALID_PEND_STATE         */
                                       /*    AP_PEND_POST_STATE             */
                                       /*    AP_SEND_RECEIVE_STATE          */
                                       /*    AP_SEND_ONLY_STATE             */
                                       /*    AP_RECEIVE_ONLY_STATE          */
    UCHAR  session_id_len;             /* session identifier length         */
    UCHAR  session_id[8];              /* session identifier                */
    UCHAR reserv7[17];                 /* Reserved                          */
} GET_ATTRIBUTES;


typedef struct prepare_to_receive {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_PREPARE_TO_RECEIVE        */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR ptr_type;                    /* Type                              */
                                       /*    AP_SYNC_LEVEL                  */
                                       /*    AP_FLUSH                       */
                                       /*    AP_P_TO_R_CONFIRM              */
    UCHAR locks;                       /* Locks                             */
                                       /*    AP_SHORT                       */
                                       /*    AP_LONG                        */
    UCHAR reserv3[18];                 /* Reserved                          */

} PREPARE_TO_RECEIVE;


typedef struct receive_and_wait {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_RECEIVE_AND_WAIT          */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    USHORT what_rcvd;                  /* What Received                     */
                                       /*    AP_NONE                        */
                                       /*    AP_DATA                        */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_INCOMPLETE             */
                                       /*    AP_SEND                        */
                                       /*    AP_CONFIRM_WHAT_RCVD           */
                                       /*    AP_CONFIRM_SEND                */
                                       /*    AP_CONFIRM_DEALLOCATE          */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_COMPLETE_CONFIRM       */
                                       /*    AP_DATA_COMPLETE_CONFIRM_DEALL */
                                       /*    AP_DATA_COMPLETE_CONFIRM_SEND  */
                                       /*    AP_DATA_COMPLETE_SEND          */
                                       /*    AP_DATA_CONFIRM                */
                                       /*    AP_DATA_CONFIRM_DEALLOCATE     */
                                       /*    AP_DATA_CONFIRM_SEND           */
                                       /*    AP_DATA_SEND                   */
                                       /*    AP_PS_HEADER                   */
                                       /*    AP_PS_HEADER_SEND              */
    /* Note:  what_rcvd consists of two bytes for indicating status and     */
    /* data received.  If rtn_status is set, one bit from each byte may be  */
    /* set.  You can 'AND' the appropriate constant with the what_rcvd      */
    /* field to test for either status or data received.                    */
    UCHAR reserv3[2];                  /* Reserved                          */
    UCHAR rtn_status;                  /* Return Status With Data           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR fill;                        /* Fill                              */
                                       /*    AP_BUFFER                      */
                                       /*    AP_LL                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    ULONG max_len;                     /* Max Length (0-65535)              */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv4[16];                 /* Reserved                          */
} RECEIVE_AND_WAIT;


typedef struct receive_immediate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_RECEIVE_IMMEDIATE         */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    USHORT what_rcvd;                  /* What Received                     */
                                       /*    AP_NONE                        */
                                       /*    AP_DATA                        */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_INCOMPLETE             */
                                       /*    AP_SEND                        */
                                       /*    AP_CONFIRM_WHAT_RCVD           */
                                       /*    AP_CONFIRM_SEND                */
                                       /*    AP_CONFIRM_DEALLOCATE          */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_COMPLETE_CONFIRM       */
                                       /*    AP_DATA_COMPLETE_CONFIRM_DEALL */
                                       /*    AP_DATA_COMPLETE_CONFIRM_SEND  */
                                       /*    AP_DATA_COMPLETE_SEND          */
                                       /*    AP_DATA_CONFIRM                */
                                       /*    AP_DATA_CONFIRM_DEALLOCATE     */
                                       /*    AP_DATA_CONFIRM_SEND           */
                                       /*    AP_DATA_SEND                   */
                                       /*    AP_PS_HEADER                   */
                                       /*    AP_PS_HEADER_SEND              */
    /* Note:  what_rcvd consists of two bytes for indicating status and     */
    /* data received.  If rtn_status is set, one bit from each byte may be  */
    /* set.  You can 'AND' the appropriate constant with the what_rcvd      */
    /* field to test for either status or data received.                    */
    UCHAR reserv5[2];                  /* Reserved                          */
    UCHAR rtn_status;                  /* Return Status With Data           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR fill;                        /* Fill                              */
                                       /*    AP_BUFFER                      */
                                       /*    AP_LL                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    ULONG max_len;                     /* Max Length (0-65535)              */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv4[16];                 /* Reserved                          */
} RECEIVE_IMMEDIATE;


typedef struct receive_expedited_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_RECEIVE_EXPEDITED_DATA    */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR return_control;              /* Return Control                    */
                                       /*    AP_WHEN_EXPEDITED_DATA_RECEIVED*/
                                       /*    AP_IMMEDIATE                   */
    UCHAR reserv3[3];                  /* Reserved                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv4[2];                  /* Reserved                          */
    ULONG max_len;                     /* Max Length (0-86))                */
    ULONG dlen;                        /* Data Length (0-86)                */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv5[16];                 /* Reserved                          */
} RECEIVE_EXPEDITED_DATA;


typedef struct request_to_send {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_REQUEST_TO_SEND           */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} REQUEST_TO_SEND;


typedef struct send_conversation {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_SEND_CONVERSATION         */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR reserv3[8];                  /* Reserved                          */
    UCHAR rtn_ctl;                     /* Return Control                    */
                                       /*    AP_WHEN_SESSION_ALLOCATED      */
                                       /*    AP_IMMEDIATE                   */
                                       /*    AP_WHEN_SESSION_FREE           */
                                       /*    AP_WHEN_CONV_GROUP_ALLOC       */
                                       /*    AP_WHEN_CONWINNER_ALLOC        */
    UCHAR reserv4[3];                  /* Reserved                          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR security;                    /* Security                          */
                                       /*    AP_NONE                        */
                                       /*    AP_SAME                        */
                                       /*    AP_PGM                         */
    UCHAR reserv5[11];                 /* Reserved                          */
    UCHAR pwd[10];                     /* Password (EBCDIC type AE)         */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG pip_dlen;                    /* PIP Data Length (0-32767)         */
                                       /*    (0 if no pip data)             */
    UCHAR * pip_dptr;                  /* PIP Data address                  */
    UCHAR reserv6;                     /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv7[34];                 /* Reserved                          */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR crypto;                      /* selective encryption option       */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR  reserv8[19];                /* Reserved                          */
} SEND_CONVERSATION;


typedef struct send_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_SEND_DATA                 */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[2];                  /* Reserve                           */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR type;                        /* Type                              */
                                       /*    AP_NONE                        */
                                       /*    AP_SEND_DATA_FLUSH             */
                                       /*    AP_SEND_DATA_CONFIRM           */
                                       /*    AP_SEND_DATA_P_TO_R_FLUSH      */
                                       /*    AP_SEND_DATA_P_TO_R_CONFIRM    */
                                       /*    AP_SEND_DATA_P_TO_R_SYNC_LEVEL */
                                       /*    AP_SEND_DATA_DEALLOC_FLUSH     */
                                       /*    AP_SEND_DATA_DEALLOC_SYNC_LEVEL*/
                                       /*    AP_SEND_DATA_DEALLOC_ABEND     */
    UCHAR reserv4;                     /* Reserved                          */
    UCHAR crypto;                      /* selective encryption option       */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR  reserv5[17];                /* Reserved                          */
} SEND_DATA;


typedef struct send_error {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_SEND_ERROR                */
    UCHAR opext;                       /* Verb extension code               */
                                       /* Calculate value using             */
                                       /*  instructions in APPCDEF.H -      */
                                       /*   Include AP_EXTENDED_VERB        */
                                       /*   in the calculated value.        */
                                       /*   May or may not include          */
                                       /*    AP_FDX_VERB and/or             */
                                       /*    AP_NONBLOCKING_VERB.           */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request To Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR err_type;                    /* Type                              */
                                       /*    AP_PROG                        */
                                       /*    AP_SVC                         */
    UCHAR err_dir;                     /* Error Direction                   */
                                       /*    AP_RCV_DIR_ERROR               */
                                       /*    AP_SEND_DIR_ERROR              */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    ULONG log_dlen;                    /* Log Data Length (0 to 32767)      */
                                       /*    (0 if no log data)             */
    UCHAR * log_dptr;                  /* Log Data address                  */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR reserv3[16];                 /* Reserved                          */
} SEND_ERROR;


typedef struct send_expedited_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_SEND_EXPEDITED_DATA       */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_NON_BLOCKING_BASIC          */
                                       /*    AP_FDX_BLOCKING_BASIC          */
                                       /*    AP_FDX_NONBLOCKING_BASIC       */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[2];                  /* Reserve                           */
    ULONG dlen;                        /* Data Length (1-86)                */
    UCHAR * dptr;                      /* Data PTR                          */
    UCHAR reserv4[16];                 /* Reserved                          */
} SEND_EXPEDITED_DATA;


typedef struct test_rts {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_B_TEST_RTS                  */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_BASIC_CONVERSATION          */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv4[16];                 /* Reserved                          */
} TEST_RTS;


/****************************************************************************/
/* Transaction Programming Interface - Mapped Conversation Verbs            */
/****************************************************************************/

typedef struct mc_allocate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_ALLOCATE                  */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3;                     /* Reserved                          */
    UCHAR sync_level;                  /* Sync Level                        */
                                       /*    AP_NONE                        */
                                       /*    AP_CONFIRM                     */
                                       /*    AP_SYNCPT                      */
    UCHAR reserv4[2];                  /* Reserved                          */
    UCHAR rtn_ctl;                     /* Return Control                    */
                                       /*    AP_WHEN_SESSION_ALLOCATED      */
                                       /*    AP_IMMEDIATE                   */
                                       /*    AP_WHEN_SESSION_FREE           */
                                       /*    AP_WHEN_CONV_GROUP_ALLOC       */
                                       /*    AP_WHEN_CONWINNER_ALLOC        */
    UCHAR conversation_style;          /* Conversation Style                */
                                       /*    AP_TWO_WAY_ALTERNATE           */
                                       /*    AP_TWO_WAY_SIMULTANEOUS        */
    UCHAR reserv5[2];                  /* Reserved                          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR security;                    /* Security                          */
                                       /*    AP_NONE                        */
                                       /*    AP_SAME                        */
                                       /*    AP_PGM                         */
    UCHAR reserv6[11];                 /* Reserved                          */
    UCHAR pwd[10];                     /* Password (EBCDIC type AE)         */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG pip_dlen;                    /* PIP Data Length                   */
                                       /*    (0 if no pip data)             */
    UCHAR * pip_dptr;                  /* PIP Data                          */
    UCHAR reserv7;                     /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv8[34];                 /* Reserved                          */
} MC_ALLOCATE;


typedef struct mc_confirm {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_CONFIRM                   */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[18];                 /* Reserved                          */
} MC_CONFIRM;


typedef struct mc_confirmed {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_CONFIRMED                 */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} MC_CONFIRMED;


typedef struct mc_deallocate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_DEALLOCATE                */
    UCHAR opext;                       /* Verb extension code               */
                                       /* Calculate value using             */
                                       /*  instructions in APPCDEF.H -      */
                                       /*   Include AP_EXTENDED_VERB        */
                                       /*     and   AP_MAPPED_VERB          */
                                       /*   in the calculated value.        */
                                       /*   May or may not include          */
                                       /*    AP_FDX_VERB and/or             */
                                       /*    AP_NONBLOCKING_VERB.           */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR dealloc_type;                /* Type of Deallocate                */
                                       /*    AP_SYNC_LEVEL                  */
                                       /*    AP_FLUSH                       */
                                       /*    AP_ABEND_PROG                  */
                                       /*    AP_CANCEL                      */
    UCHAR reserv3[10];                 /* Reserved                          */
    ULONG sense_data;                  /* Optional Sense data for           */
                                       /*  type AP_ABEND_PROG               */
    UCHAR reserv4[16];                 /* Reserved                          */
} MC_DEALLOCATE;


typedef struct mc_flush {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_FLUSH                     */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} MC_FLUSH;


typedef struct mc_get_attributes {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_GET_ATTRIBUTES            */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv4;                     /* Reserved                          */
    UCHAR sync_level;                  /* Sync Level                        */
                                       /*    AP_NONE                        */
                                       /*    AP_CONFIRM                     */
                                       /*    AP_SYNCPT                      */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR net_name[8];                 /* NETID (EBCDIC type A)             */
    UCHAR lu_name[8];                  /* LU Name (EBCDIC type A)           */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII)                  */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR plu_un_name[8];              /* Partner LU Uninterpreted Name     */
                                       /*    (EBCDIC type AE)               */
    UCHAR reserv5[2];                  /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv6;                     /* Reserved                          */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    UCHAR  conv_corr_len;              /* Coversation correlator length     */
    UCHAR  conv_corr[8];               /* Conversation correlator           */
    UCHAR conversation_state;          /* Conversation State                */
                                       /*    AP_RESET_STATE                 */
                                       /*    AP_SEND_STATE                  */
                                       /*    AP_RECEIVE_STATE               */
                                       /*    AP_CONFIRM_STATE               */
                                       /*    AP_CONFIRM_SEND_STATE          */
                                       /*    AP_CONFIRM_DEALL_STATE         */
                                       /*    AP_PEND_DEALL_STATE            */
                                       /*    AP_END_CONV_STATE              */
                                       /*    AP_TP_VALID_PEND_STATE         */
                                       /*    AP_PEND_POST_STATE             */
                                       /*    AP_SEND_RECEIVE_STATE          */
                                       /*    AP_SEND_ONLY_STATE             */
                                       /*    AP_RECEIVE_ONLY_STATE          */
    UCHAR  session_id_len;             /* session identifier length         */
    UCHAR  session_id[8];              /* session identifier                */
    UCHAR  reserv7[17];                /* Reserved                          */
} MC_GET_ATTRIBUTES;


typedef struct mc_prepare_to_receive {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_PREPARE_TO_RECEIVE        */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR ptr_type;                    /* Type                              */
                                       /*    AP_SYNC_LEVEL                  */
                                       /*    AP_FLUSH                       */
                                       /*    AP_P_TO_R_CONFIRM              */
    UCHAR locks;                       /* Locks                             */
                                       /*    AP_SHORT                       */
                                       /*    AP_LONG                        */
    UCHAR reserv3[18];                 /* Reserved                          */

} MC_PREPARE_TO_RECEIVE;


typedef struct mc_receive_and_wait {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_RECEIVE_AND_WAIT          */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    USHORT what_rcvd;                  /* What Received                     */
                                       /*    AP_NONE                        */
                                       /*    AP_DATA                        */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_INCOMPLETE             */
                                       /*    AP_UC_DATA_COMPLETE            */
                                       /*    AP_UC_DATA_INCOMPLETE          */
                                       /*    AP_SEND                        */
                                       /*    AP_CONFIRM_WHAT_RCVD           */
                                       /*    AP_CONFIRM_SEND                */
                                       /*    AP_CONFIRM_DEALLOCATE          */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_COMPLETE_CONFIRM       */
                                       /*    AP_DATA_COMPLETE_CONFIRM_DEALL */
                                       /*    AP_DATA_COMPLETE_CONFIRM_SEND  */
                                       /*    AP_DATA_COMPLETE_SEND          */
                                       /*    AP_UC_DATA_COMPLETE_CFM        */
                                       /*    AP_UC_DATA_COMPLETE_CFM_DEALL  */
                                       /*    AP_UC_DATA_COMPLETE_CFM_SEND   */
                                       /*    AP_UC_DATA_COMPLETE_SEND       */
                                       /*    AP_DATA_CONFIRM                */
                                       /*    AP_DATA_CONFIRM_DEALLOCATE     */
                                       /*    AP_DATA_CONFIRM_SEND           */
                                       /*    AP_DATA_SEND                   */
                                       /*    AP_PS_HEADER                   */
                                       /*    AP_PS_HEADER_SEND              */
    /* Note:  what_rcvd consists of two bytes for indicating status and     */
    /* data received.  If rtn_status is set, one bit from each byte may be  */
    /* set.  You can 'AND' the appropriate constant with the what_rcvd      */
    /* field to test for either status or data received.                    */
    UCHAR reserv5[2];                  /* Reserved                          */
    UCHAR rtn_status;                  /* Return Status With Data           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3;                     /* Reserved                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    ULONG max_len;                     /* Max Length (0-65535)              */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv4[16];                 /* Reserved                          */
} MC_RECEIVE_AND_WAIT;


typedef struct mc_receive_immediate {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_RECEIVE_IMMEDIATE         */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    USHORT what_rcvd;                  /* What Received                     */
                                       /*    AP_NONE                        */
                                       /*    AP_DATA                        */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_INCOMPLETE             */
                                       /*    AP_UC_DATA_COMPLETE            */
                                       /*    AP_UC_DATA_INCOMPLETE          */
                                       /*    AP_SEND                        */
                                       /*    AP_CONFIRM_WHAT_RCVD           */
                                       /*    AP_CONFIRM_SEND                */
                                       /*    AP_CONFIRM_DEALLOCATE          */
                                       /*    AP_DATA_COMPLETE               */
                                       /*    AP_DATA_COMPLETE_CONFIRM       */
                                       /*    AP_DATA_COMPLETE_CONFIRM_DEALL */
                                       /*    AP_DATA_COMPLETE_CONFIRM_SEND  */
                                       /*    AP_DATA_COMPLETE_SEND          */
                                       /*    AP_UC_DATA_COMPLETE_CFM        */
                                       /*    AP_UC_DATA_COMPLETE_CFM_DEALL  */
                                       /*    AP_UC_DATA_COMPLETE_CFM_SEND   */
                                       /*    AP_UC_DATA_COMPLETE_SEND       */
                                       /*    AP_DATA_CONFIRM                */
                                       /*    AP_DATA_CONFIRM_DEALLOCATE     */
                                       /*    AP_DATA_CONFIRM_SEND           */
                                       /*    AP_DATA_SEND                   */
                                       /*    AP_PS_HEADER                   */
                                       /*    AP_PS_HEADER_SEND              */
    /* Note:  what_rcvd consists of two bytes for indicating status and     */
    /* data received.  If rtn_status is set, one bit from each byte may be  */
    /* set.  You can 'AND' the appropriate constant with the what_rcvd      */
    /* field to test for either status or data received.                    */
    UCHAR reserv6[2];                  /* Reserved                          */
    UCHAR rtn_status;                  /* Return Status With Data           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv4;                     /* Reserved                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    ULONG max_len;                     /* Max Length (0-65535)              */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv5[16];                 /* Reserved                          */
} MC_RECEIVE_IMMEDIATE;


typedef struct mc_receive_expedited_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_RECEIVE_EXPEDITED_DATA    */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR return_control;              /* Return Control                    */
                                       /*    AP_WHEN_EXPEDITED_DATA_RECEIVED*/
                                       /*    AP_IMMEDIATE                   */
    UCHAR reserv3[3];                  /* Reserved                          */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv4[2];                  /* Reserved                          */
    ULONG max_len;                     /* Max Length (0-86)                 */
    ULONG dlen;                        /* Data Length (0-86)                */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR reserv5[16];                 /* Reserved                          */
} MC_RECEIVE_EXPEDITED_DATA;


typedef struct mc_request_to_send {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_REQUEST_TO_SEND           */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv3[16];                 /* Reserved                          */
} MC_REQUEST_TO_SEND;


typedef struct mc_send_conversation {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_SEND_CONVERSATION         */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR reserv3[8];                  /* Reserved                          */
    UCHAR rtn_ctl;                     /* Return Control                    */
                                       /*    AP_WHEN_SESSION_ALLOCATED      */
                                       /*    AP_IMMEDIATE                   */
                                       /*    AP_WHEN_SESSION_FREE           */
                                       /*    AP_WHEN_CONV_GROUP_ALLOC       */
                                       /*    AP_WHEN_CONWINNER_ALLOC        */
    UCHAR reserv4[3];                  /* Reserved                          */
    ULONG conv_group_id;               /* Conversation Group ID             */
    ULONG sense_data;                  /* Sense Data                        */
    UCHAR plu_alias[8];                /* Partner LU Alias (ASCII)          */
    UCHAR mode_name[8];                /* Mode Name (EBCDIC type A)         */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR security;                    /* Security                          */
                                       /*    AP_NONE                        */
                                       /*    AP_SAME                        */
                                       /*    AP_PGM                         */
    UCHAR reserv5[11];                 /* Reserved                          */
    UCHAR pwd[10];                     /* Password (EBCDIC type AE)         */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    ULONG pip_dlen;                    /* PIP Data Length (0-32767)         */
                                       /*    (0 if no pip data)             */
    UCHAR * pip_dptr;                  /* PIP Data address                  */
    UCHAR reserv6;                     /* Reserved                          */
    UCHAR fqplu_name[17];              /* Fully Qualified Partner LU Name   */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv7[34];                 /* Reserved                          */
    ULONG dlen;                        /* Data Length (0-65535)             */
    UCHAR * dptr;                      /* Data buffer address               */
    UCHAR crypto;                      /* selective encryption option       */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR  reserv8[19];                /* Reserved                          */
} MC_SEND_CONVERSATION;


typedef struct mc_send_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_SEND_DATA                 */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[2];                  /* Reserved                          */
    ULONG dlen;                        /* Data Length                       */
    UCHAR * dptr;                      /* Data PTR                          */
    UCHAR type;                        /* Type                              */
                                       /*    AP_NONE                        */
                                       /*    AP_SEND_DATA_FLUSH             */
                                       /*    AP_SEND_DATA_CONFIRM           */
                                       /*    AP_SEND_DATA_P_TO_R_FLUSH      */
                                       /*    AP_SEND_DATA_P_TO_R_CONFIRM    */
                                       /*    AP_SEND_DATA_P_TO_R_SYNC_LEVEL */
                                       /*    AP_SEND_DATA_DEALLOC_FLUSH     */
                                       /*    AP_SEND_DATA_DEALLOC_SYNC_LEVEL*/
                                       /*    AP_SEND_DATA_DEALLOC_ABEND     */
    UCHAR user_control_data;           /* User Control Data                 */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR crypto;                      /* selective encryption option       */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv4[17];                 /* Reserved                          */
} MC_SEND_DATA;


typedef struct mc_send_error {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_SEND_ERROR                */
    UCHAR opext;                       /* Verb extension code               */
                                       /* Calculate value using             */
                                       /*  instructions in APPCDEF.H -      */
                                       /*   Include AP_EXTENDED_VERB        */
                                       /*     and   AP_MAPPED_VERB          */
                                       /*   in the calculated value.        */
                                       /*   May or may not include          */
                                       /*    AP_FDX_VERB and/or             */
                                       /*    AP_NONBLOCKING_VERB.           */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request To Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3;                     /* Reserved                          */
    UCHAR err_dir;                     /* Error Direction                   */
                                       /*    AP_RCV_DIR_ERROR               */
                                       /*    AP_SEND_DIR_ERROR              */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv4[8];                  /* Reserved                          */
    /* The following fields make up the extension added to this verb.       */

    ULONG sense_data;                  /* Optional Sense data               */
    UCHAR reserv5[16];                 /* Reserved                          */
} MC_SEND_ERROR;


typedef struct mc_send_expedited_data {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_SEND_EXPEDITED_DATA       */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
                                       /*    AP_NON_BLOCKING_MAPPED         */
                                       /*    AP_FDX_BLOCKING_MAPPED         */
                                       /*    AP_FDX_NONBLOCKING_MAPPED      */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR rts_rcvd;                    /* Request to Send Received          */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR expd_data_rcvd;              /* Expedited Data Received           */
                                       /*    AP_NO                          */
                                       /*    AP_YES                         */
    UCHAR reserv3[2];                  /* Reserved                          */
    ULONG dlen;                        /* Data Length (1-86)                */
    UCHAR * dptr;                      /* Data PTR                          */
    UCHAR reserv4[16];                 /* Reserved                          */
} MC_SEND_EXPEDITED_DATA;


typedef struct mc_test_rts {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_M_TEST_RTS                  */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_MAPPED_CONVERSATION         */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv4[16];                 /* Reserved                          */
} MC_TEST_RTS;


 /***************************************************************************/
 /*       Type Independent Conversation Interface                           */
 /***************************************************************************/

typedef struct get_type {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_GET_TYPE                    */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR conv_type;                   /* Conversation type                 */
                                       /*    AP_BASIC_CONVERSATION          */
                                       /*    AP_MAPPED_CONVERSATION         */
    UCHAR conv_style;                  /* Conversation style                */
                                       /*    AP_TWO_WAY_ALTERNATE           */
                                       /*    AP_TWO_WAY_SIMULTANEOUS        */
    UCHAR reserv4[34];                 /* Reserved                          */
} GET_TYPE;


typedef struct get_tp_properties {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_GET_TP_PROPERTIES           */
    UCHAR opext;                       /* Verb extension code               */
                                       /*    AP_EXTENDED_VERB               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR tp_name[64];                 /* TP Name (EBCDIC type AE)          */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII)                  */
    LUW_ID_OVERLAY luw_id;             /* Logical Unit of Work ID           */
    UCHAR fqlu_name[17];               /* Fully Qualified LU Name           */
                                       /*    0 or 1-17 bytes (EBCDIC type A)*/
    UCHAR reserv4[10];                 /* Reserved                          */
    UCHAR user_id[10];                 /* User ID (EBCDIC type AE)          */
    UCHAR reserv5[3];                  /* Reserved                          */
    LUW_ID_OVERLAY protected_luw_id;   /* Protected (for syncpt)            */
                                       /*  Logical Unit of Work ID          */
    UCHAR reserv6[32];                 /* Reserved                          */
} GET_TP_PROPERTIES;


typedef struct set_luwid {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_SET_LUWID                   */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    UCHAR type;                        /* Type of LUWID                     */
                                       /*    AP_PROTECTED                   */
                                       /*    AP_UNPROTECTED                 */
    UCHAR reserv4[3];                  /* Reserved                          */
    LUW_ID_OVERLAY luw_id;             /* Logical Unit of Work ID           */
    UCHAR reserv5[34];                 /* Reserved                          */
} SET_LUWID;


typedef struct set_implied_forget {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_SET_IMPLIED_FORGET          */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR event_id[4];                 /* Implied Forget identifier         */
    UCHAR reserv4[32];                 /* Reserved                          */
} SET_IMPLIED_FORGET;


typedef struct free_conversation  {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_FREE_CONVERSATION           */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR tp_id[8];                    /* TP ID                             */
    ULONG conv_id;                     /* Conversation ID                   */
    UCHAR reserv4[32];                 /* Reserved                          */
} FREE_CONVERSATION;


typedef struct register_queue     {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_REGISTER_QUEUE              */
    UCHAR opext;                       /* Verb extension code               */
    UCHAR reserv1[5];                  /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR reserv2[2];                  /* Reserved                          */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    UCHAR reserv3[4];                  /* Reserved                          */
    UCHAR queue_name[80];              /* Queue name (ASCII)                */
    UCHAR provider_name[16];           /* Service provider name (ASCII)     */
    UCHAR * shared_pool;               /* Shared memory pool address        */
    UCHAR provider_type;               /* Type of service provider          */
                                       /*    AP_SYNCPT_PROVIDER             */
                                       /*    AP_REMOTE_TP_PROVIDER          */
                                       /*    (AP_REMOTE_TP_PROVIDER         */
                                       /*     | AP_SYNCPT_PROVIDER)         */
    UCHAR operation_type;              /* Operation type                    */
                                       /*    AP_ENABLE_SERVICE              */
                                       /*    AP_DISABLE_SERVICE             */
    UCHAR lu_name[8];                  /* Local LU name to be enabled       */
                                       /*  for service associated           */
                                       /*  with the queue.                  */
    UCHAR resync_tp_filespec[80];      /* File spec of resync TP            */
                                       /* associated with this              */
                                       /* provider (if SYNCPT type)         */
    UCHAR lu_alias[8];                 /* LU Alias (ASCII) if               */
                                       /* lu_name (EBCDIC) doesn't match    */
    UCHAR reserv4[26];                 /* Reserved                          */
} REGISTER_QUEUE;


typedef struct register_nb_queues {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_REGISTER_NB_QUEUES          */
    UCHAR  reserv1[6];                 /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR  reserv2[2];                 /* Reserved                          */
    ULONG  secondary_rc;               /* Secondary RETURN_CODE             */
    UCHAR  reserv3[4];                 /* Reserved                          */
    ULONG  queue_request_id;           /* Queue Request ID                  */
    UCHAR  snd_q_name[81];             /* OS/2 Queue Name (ASCIIZ)          */
    UCHAR  reserv4;                    /* Reserved                          */
    UCHAR  rcv_q_name[81];             /* OS/2 Queue Name (ASCIIZ)          */
    UCHAR  reserv5[33];                /* Reserved                          */
} REGISTER_NB_QUEUES;

typedef struct register_memory_object {
    USHORT opcode;                     /* Verb operation code               */
                                       /*    AP_REGISTER_MEMORY_OBJECT      */
    UCHAR  reserv1[6];                 /* Reserved                          */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR  reserv2[2];                 /* Reserved                          */
    ULONG  secondary_rc;               /* Secondary RETURN_CODE             */
    UCHAR  reserv3[4];                 /* Reserved                          */
    PVOID  object_address;             /* Object address                    */
    ULONG  reg_unreg_type;             /* Register/Unregister flag          */
                                       /*    AP_REGISTER_MEMORY             */
                                       /*    AP_UNREGISTER_MEMORY           */
    UCHAR  reserv4[32];                /* Reserved                          */
} REGISTER_MEMORY_OBJECT;


/* This structure should only be used for migration of 16 bit code */
/* to 32-bit code.  Otherwise the structure, APPC32_HEADER,  given */
/* near the top of the file should be used.  That structures uses  */
/* masks, rather than explicit bit fields, to access the opcode    */
/* extension area, and is much more reliable for code portability. */

typedef struct appc_hdr {
    USHORT opcode;                     /* Verb operation code               */
    unsigned int opext_mc       : 1;   /* Mapped Conversation               */
    unsigned int opext_nb       : 1;   /* Non Blocking                      */
    unsigned int opext_fdx      : 1;   /* Full Duplex                       */
    unsigned int                : 1;   /* Reserved                          */
    unsigned int                : 1;   /* Reserved                          */
    unsigned int                : 1;   /* Reserved                          */
    unsigned int opext_ic       : 1;   /* Incomplete                        */
    unsigned int                : 1;   /* Reserved                          */
    unsigned int reserv2        : 8;   /* Reserved                          */
    UCHAR    reserv1[4];               /* Reserverd                         */
    USHORT primary_rc;                 /* Primary RETURN_CODE               */
    UCHAR    reserv3[2];               /* Reserverd                         */
    ULONG secondary_rc;                /* Secondary RETURN_CODE             */
    WAIT_OBJECT wait;                  /* Wait control strucutre            */
} APPC_HDR;

#endif

#ifdef __cplusplus
  }
#endif
